'''
Created on 13.08.2010

@author: gsu
'''

from enum import Enum
from artixds.campaign import DiscountCampaignGroup, DiscountCampaign

class DiscountSystemType(Enum):
    DISCOUNT = 'discount'
    BONUS = 'bonus'
    BONUSPAY = 'bonuspay'

class DiscountSystem(object):
    rootCampaignGroup = DiscountCampaignGroup()
    manualCampaigns = []
    groupsAndCampaignsPriorities = []
    def __init__(self, rootCampaignGroup=None, type=DiscountSystemType.DISCOUNT.value, manualCampaigns=[], number=1, groupsAndCampaignsPriorities=[]):
        self.rootCampaignGroup = rootCampaignGroup
        self.type = type
        self.manualCampaigns = manualCampaigns
        self.number = number
        self.groupsAndCampaignsPriorities = groupsAndCampaignsPriorities

    def getManualCampaignsInfo(self, campaignGroup):
        objects = campaignGroup.discountCampaigns + campaignGroup.childGroups
        result = []
        for obj in objects:
            if isinstance(obj, DiscountCampaign):
                if hasattr(obj,'manualActivate') and obj.manualActivate and obj.isActive():
                    campaignInfo = {}
                    campaignInfo['id'] = obj.id
                    campaignInfo['name'] = obj.name
                    campaignInfo['beginDate'] = obj.beginDate
                    campaignInfo['endDate'] = obj.endDate
                    campaignInfo['beginTime'] = obj.beginTime
                    campaignInfo['endTime'] = obj.endTime
                    campaignInfo['daysOfWeek'] = obj.daysOfWeek
                    campaignInfo['checkedTags'] = obj.checkedTags
                    campaignInfo['tagsIsAllowed'] = obj.tagsIsAllowed
                    if obj.discounts:
                        campaignInfo['discountId'] = obj.discounts[0].id
                    result.append(campaignInfo)
            elif isinstance(obj, DiscountCampaignGroup):
                result.extend(self.getManualCampaignsInfo(obj))
        return result
    
    def loadManualCampaigns(self):
        self.manualCampaigns = self.getManualCampaignsInfo(self.rootCampaignGroup)

    def getCampaignsPriorities(self, campaignGroup, parentPriority):
        objects = campaignGroup.discountCampaigns + campaignGroup.childGroups
        priorities = set()
        for obj in objects:
            if isinstance(obj, DiscountCampaign):
                if hasattr(obj,'priority'):
                    if (obj.priority is None and parentPriority is not None):
                        obj.priority = parentPriority
                    priorities.add(obj.priority)
            elif isinstance(obj, DiscountCampaignGroup):
                groupPriority = parentPriority
                if hasattr(obj,'priority'):
                    if (obj.priority is not None):
                        groupPriority = obj.priority
                    else:
                        obj.priority = parentPriority
                priorities = priorities.union(self.getCampaignsPriorities(obj, groupPriority))
        return priorities

    def getGroupsAndCampaignsPriorities(self, campaignGroup):
        priorities = set()
        priority = None
        if hasattr(campaignGroup,'priority'):
            priority = campaignGroup.priority
        priorities = priorities.union(self.getCampaignsPriorities(campaignGroup, priority))
        priorities = list(priorities)
        priorities.sort(key=lambda priority: -1 if priority is None else priority)
        if not priorities:
            priorities.append(None)
        return priorities

    def loadGroupsAndCampaignsPriorities(self):
        self.groupsAndCampaignsPriorities = self.getGroupsAndCampaignsPriorities(self.rootCampaignGroup)
