# -*- coding: utf-8 -*-

from config import Config
from artixds.domain import DiscountSystem
from sqlalchemy.orm import mapper
from sqlalchemy.orm import clear_mappers
from artixds.mapper import DiscountSystemData, discountSystemTable
from sqlalchemy.orm import sessionmaker
from sqlalchemy import create_engine
import yaml
from yaml import CLoader as Loader

dbName = "dictionaries"


def loadDiscountSystem(session, discSys, secondDiscSys, bonSys, bon1Sys, update_time=None):
    resultTime = None
    discountSystems = session.query(DiscountSystemData).all()
    for sys in discountSystems:
        if not update_time or sys.update_time != update_time:
            if sys.discsystem:
                discSys.rootCampaignGroup = yaml.load(sys.discsystem, Loader)
                discSys.loadManualCampaigns()
                discSys.loadGroupsAndCampaignsPriorities()
                discSys.number = 1
            else:
                discSys.rootCampaignGroup = None
                discSys.manualCampaigns = []
            if sys.seconddiscsystem:
                secondDiscSys.rootCampaignGroup = yaml.load(sys.seconddiscsystem, Loader)
                secondDiscSys.loadManualCampaigns()
                secondDiscSys.loadGroupsAndCampaignsPriorities()
                secondDiscSys.number = 2
            else:
                secondDiscSys.rootCampaignGroup = None
                secondDiscSys.manualCampaigns = []
            if sys.bonusearnsystem:
                bonSys.rootCampaignGroup = yaml.load(sys.bonusearnsystem, Loader)
            else:
                bonSys.rootCampaignGroup = None
                bonSys.manualCampaigns = []
            if sys.bonuslimitsystem:
                bon1Sys.rootCampaignGroup = yaml.load(sys.bonuslimitsystem, Loader)
            else:
                bon1Sys.rootCampaignGroup = None
                bon1Sys.manualCampaigns = []
            resultTime = sys.update_time
    return resultTime if resultTime else update_time


def createEngine():
    host = Config().getString("MysqlServer", "host", "localhost")
    port = Config().getString("MysqlServer", "port", "3306")
    user = Config().getString("MysqlServer", "user", "root")
    password = Config().getString("MysqlServer", "password", "")

    global dbName
    dbName = Config().getString("MysqlServer", "dictionariesDbName", "dictionaries")

    return create_engine('mysql+pymysql://%s:%s@%s:%s/%s?charset=utf8mb4' % (user, password, host, port, dbName), echo=True)


def createSession():
    clear_mappers()
    mapper(DiscountSystemData, discountSystemTable)
    engine = createEngine()
    Session = sessionmaker(autoflush=True, autocommit=True)
    Session.configure(bind=engine)
    return Session()


def getDiscountUpdateTime(session):
    currentDbName = Config().getString("MysqlServer", "dictionariesDbName", "dictionaries")
    if currentDbName != dbName:
        return None, createSession()

    discountSystems = session.query(DiscountSystemData.update_time).all()
    for sys in discountSystems:
        return sys.update_time, session
    return None, session


def loadDiscountSystemLegacy():
    session = createSession()
    discSys, bonSys, bonpSys = None, None, None
    discountSystems = session.query(DiscountSystemData).all()
    for sys in discountSystems:
        discSys = DiscountSystem()
        discSys.rootCampaignGroup = yaml.load(sys.discsystem)
    return discSys, bonSys, bonpSys


def loadDiscountSystemSimple():
    session = createSession()
    discSys, bonSys, bonpSys = None, None, None
    discountSystems = session.query(DiscountSystemData).all()
    for sys in discountSystems:
        discSys = DiscountSystem()
        discSys.rootCampaignGroup = yaml.load(sys.discsystem)
    return discSys, bonSys, bonpSys
